/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.changetracking;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.Content;
import ro.sync.ecss.markers.MarkerBase;
import ro.sync.ecss.markers.ContentMarker;


/**
 * This object marks a change in the Author (insert or delete).
 */
public class ChangeMarker extends ContentMarker{
  
  /**
   * Constructor.
   * 
   * @param type The persistent highlight type, 
   * one of {@link ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight.PersistentHighlightType#CHANGE_DELETE} or 
   * {@link ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight.PersistentHighlightType#CHANGE_INSERT}
   */
  public ChangeMarker(PersistentHighlightType type) {
    super(type);
  }
  
  /**
   * Create a fixed marker with the info from another marker.
   * 
   * @param content The author content.
   * @param marker The original marker.
   * @param startOffset The start offset
   * @param endOffset The end offset
   * @return The new marker.
   * @throws BadLocationException Thrown if it fails to create positions.
   */
  public static final ChangeMarker createChangeMarker(Content content, MarkerBase marker, int startOffset, int endOffset) throws BadLocationException {
    return createChangeMarker(content, marker, null, startOffset, endOffset);
  }
  
  /**
   * Create a fixed marker with the info from another marker.
   * 
   * @param content The author content.
   * @param markerType The marker type
   * @param startOffset The start offset
   * @param endOffset The end offset
   * @return The new marker.
   * @throws BadLocationException Thrown if it fails to create positions.
   */
  public static final ChangeMarker createChangeMarker(Content content, PersistentHighlightType markerType, int startOffset, int endOffset) throws BadLocationException {
    return createChangeMarker(content, null, markerType, startOffset, endOffset);
  }
  
  /**
   * Create a fixed marker with the info from another marker.
   * 
   * @param content The author content.
   * @param marker The original marker.
   * @param startOffset The start offset
   * @param endOffset The end offset
   * @return The new marker.
   * @throws BadLocationException Thrown if it fails to create positions.
   */
  private static final ChangeMarker createChangeMarker(Content content, MarkerBase marker, 
      PersistentHighlightType markerType, int startOffset, int endOffset) throws BadLocationException {
    ChangeMarker newMarker = new ChangeMarker(marker != null ? marker.getType() : markerType);
    if(marker != null) {
      newMarker.getProperties().putAll(marker.getProperties());
    }
    newMarker.setStartMark(content.createPosition(startOffset));
    newMarker.setEndMark(content.createPosition(endOffset));
    return newMarker;
  }
  
  /**
   * Create a fixed marker with the info from another marker.
   * 
   * @param content The author content.
   * @param marker The original marker.
   * @return The new marker.
   * @throws BadLocationException Thrown if it fails to create positions.
   */
  public static final ChangeMarker duplicate(Content content, MarkerBase marker) throws BadLocationException {
    return createChangeMarker(content, marker, marker.getStartOffset(), marker.getEndOffset());
  }
}
