/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JPanel;

import ro.sync.exml.editor.AdditionalDockableViewer;
import ro.sync.exml.editor.EditorListenerAdapter;
import ro.sync.exml.editor.EditorPageChangeEvent;
import ro.sync.exml.editor.xmleditor.XmlEditor;

/**
 * Contains an additional edit helper depending on the current editor page.
 */
class AdditionalEditHelperPanel extends JPanel {

  /**
   * The editor
   */
  private final XmlEditor editor;
  /**
   * The panel ID
   */
  private final int helperID;

  /**
   * Constructor 
   * @param editor The editor
   * @param helperID The helper panel ID
   */
  public AdditionalEditHelperPanel(XmlEditor editor, int helperID) {
    this.editor = editor;
    this.helperID = helperID;
    setLayout(new BorderLayout());
    reconfigure();
    
    editor.addEditorListener(new EditorListenerAdapter() {
      /**
       * @see ro.sync.exml.editor.EditorListenerAdapter#editorPageChanged(ro.sync.exml.editor.EditorPageChangeEvent)
       */
      @Override
      public void editorPageChanged(EditorPageChangeEvent event) {
        //Reconfigure the helper.
        reconfigure();
      }
    });
  }

  /**
   * Reconfigure its contents from the current editor page.
   */
  private void reconfigure() {
	//Remove all components.
    removeAll();
    
    //Create the new component.
    AdditionalDockableViewer dockableViewer = null;
    switch (helperID) {
      case AuthorComponentProvider.OUTLINER_PANEL_ID: 
    	//Outline
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_OUTLINER);
        break;
      case AuthorComponentProvider.ATTRIBUTES_PANEL_ID:
    	  //Attributes
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_ATTRIBUTES);
        break;
      case AuthorComponentProvider.ENTITIES_PANEL_ID: 
    	  //Entities
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_ENTITIES);
        break;
      case AuthorComponentProvider.MODEL_PANEL_ID:
    	  //Model
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_MODEL);
        break;
      case AuthorComponentProvider.ELEMENTS_PANEL_ID:
    	  //Elements
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_ELEMENTS);
        break;
      case AuthorComponentProvider.REVIEWS_PANEL_ID:
    	  //Review
        dockableViewer = editor.getCurrentPage().getAdditionalDockableViewer(
            AdditionalDockableViewer.VIEWER_ID_AUTHOR_REVIEW);
        if (dockableViewer != null) {
          dockableViewer.componentVisibleStateChanged(true);
        }
        break;
    }
    
    //And add the new component.
    if(dockableViewer != null) {
      JComponent comp = dockableViewer.getComponent();
      if(comp != null) {
        add(comp, BorderLayout.CENTER);
        revalidate();
        invalidate();
      }
      repaint();
    }
  }
}
