/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;

import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.azcheck.ui.SpellCheckOptions;
import ro.sync.ecss.common.CommonAccess;
import ro.sync.ecss.common.SACommonAccessCustomizer;
import ro.sync.ecss.dita.DITAAccess;
import ro.sync.ecss.dita.SADITAAccessCustomizer;
import ro.sync.ecss.docbook.DocbookAccess;
import ro.sync.ecss.docbook.SADocbookAccessCustomizer;
import ro.sync.ecss.extensions.api.component.ditamap.DITAMapComponentEditorManager;
import ro.sync.ecss.extensions.api.component.ditamap.DITAMapTreeComponentFactoryCustomizers;
import ro.sync.ecss.extensions.api.component.ditamap.DITAMapTreeComponentProvider;
import ro.sync.ecss.extensions.api.component.listeners.OpenURLHandler;
import ro.sync.ecss.extensions.api.component.plugins.AuthorLayoutCustomizer;
import ro.sync.ecss.extensions.api.component.plugins.AuthorPluginWorkspaceImpl;
import ro.sync.ecss.extensions.api.component.sync.ExpandedSyncInfo;
import ro.sync.ecss.extensions.api.component.sync.Resource2Synchronize;
import ro.sync.ecss.extensions.api.component.sync.ResourcesSynchronizer;
import ro.sync.ecss.images.xmlimages.mathml.MathFlowMathMLImageHandler;
import ro.sync.exml.ComponentsValidatorFactory;
import ro.sync.exml.IDEAccess;
import ro.sync.exml.OptionListeners;
import ro.sync.exml.OxygenSystemProperties;
import ro.sync.exml.SimpleSAIDEAccess;
import ro.sync.exml.editor.EditorPageConstants;
import ro.sync.exml.editor.xsdeditor.XMLSchemaVersion;
import ro.sync.exml.license.AbstractLicenseChecker;
import ro.sync.exml.license.LicenseChecker;
import ro.sync.exml.license.LicenseServerInformation;
import ro.sync.exml.license.LicenseServerInformation.LicenseServerType;
import ro.sync.exml.options.LicenseOptions;
import ro.sync.exml.options.OptionConstants;
import ro.sync.exml.options.OptionTags;
import ro.sync.exml.options.Options;
import ro.sync.exml.plugin.Plugin;
import ro.sync.exml.plugin.PluginDescriptor;
import ro.sync.exml.plugin.PluginExtension;
import ro.sync.exml.plugin.PluginManager;
import ro.sync.exml.plugin.workspace.WorkspaceAccessPluginExtension;
import ro.sync.exml.project.WorkspaceController;
import ro.sync.exml.workspace.api.standalone.InputURLChooserCustomizer;
import ro.sync.exml.workspace.api.standalone.MenuBarCustomizer;
import ro.sync.exml.workspace.api.standalone.ReferencesCustomizer;
import ro.sync.exml.workspace.api.standalone.ToolbarComponentsCustomizer;
import ro.sync.exml.workspace.api.standalone.ViewComponentCustomizer;
import ro.sync.exml.workspace.api.standalone.ditamap.TopicRefTargetInfoProvider;
import ro.sync.exml.workspace.api.util.RelativeReferenceResolver;
import ro.sync.exml.workspace.api.util.UtilAccess;
import ro.sync.exml.workspace.api.util.XMLUtilAccess;
import ro.sync.exml.workspace.impl.util.AccessUtilImpl;
import ro.sync.exml.zip.ArchiveUtilAdaptor;
import ro.sync.exml.zip.ArchiveUtilFacade;
import ro.sync.find.textcomponent.FindReplaceDialog;
import ro.sync.io.FileSystemUtil;
import ro.sync.licensemanager.FloatingLicenseCallback;
import ro.sync.licensemanager.LicenseInformation;
import ro.sync.licensemanager.LicenseInformationProvider;
import ro.sync.net.protocol.Installer;
import ro.sync.options.PersistentObject;
import ro.sync.ui.application.ApplicationDescriptorSingleton;
import ro.sync.ui.application.ApplicationOptionPane;
import ro.sync.ui.application.UnicodeFontManager;
import ro.sync.ui.application.bundle.AuthorComponentBundleDescriptor;
import ro.sync.ui.application.bundle.BundleDescriptor;
import ro.sync.util.MacClipboardUtil;
import ro.sync.util.PreferencesDirectoryDetector;
import ro.sync.util.URLUtil;

/**
 * Creates author components
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PRIVATE)
public class AuthorComponentFactory implements ReferencesCustomizer {
  
  /** 
   * Logger for logging. 
   */
  private static Logger logger = Logger.getLogger(AuthorComponentFactory.class.getName());
  
  /**
   * Name of the frameworks directory
   */
  private static final String FRAMEWORKS_DIR_NAME = "frameworks";

  /**
   * The license checker.
   */
  private static AbstractLicenseChecker licenseChecker;

  /**
   *  Used to provide the component version and release date.
   */
  private static final BundleDescriptor BUNDLE_DESCRIPTOR = new AuthorComponentBundleDescriptor();
  
  /** 
   * The singleton instance. 
   */
  private static AuthorComponentFactory instance;
  
  /**
   * The parent frame
   */
  private Frame parentFrame;
  
  /**
   * Different behavior when running from the testcases.
   */
  static boolean fromTests = false;
  
  /**
   * Customizers for the factory
   */
  private AuthorComponentFactoryCustomizers customizers = new AuthorComponentFactoryCustomizers();

  /**
   * Customizers for the factory
   */
  private DITAMapTreeComponentFactoryCustomizers ditaMapTreeCustomizers = new DITAMapTreeComponentFactoryCustomizers();
  
  /**
   * Utilities
   */
  private AccessUtilImpl accessUtil;
  
  /**
   * The plugins layout customizer.
   */
  private AuthorLayoutCustomizer pluginsLayoutCustomizer;
  
  /** 
   * Get the singleton instance. 
   * @return The singleton instance. 
   */
  public static AuthorComponentFactory getInstance() {
    if (instance == null) {
      instance = new AuthorComponentFactory();
    }
    return instance;
  }
  
  /**
   * Sets the license key. When the component is created, the license key is checked.<br>
   * Please note that this is not the recommended way to deploy the license keys. 
   * The licenses are user based, and by setting the same license key to all the 
   * users you risk of violating the license. You should take proactive 
   * actions so that the number of licensed users is not violated, or use a license server if
   * you are using a floating license model.  
   * 
   * @param licenseKey The license key text. Never null.
   */
  private void registerLicenseKey(String licenseKey) {
    // Set the license.
    LicenseOptions.getInstance().setLicenseText(licenseKey);      
  }
  
  /**
   * It is advisable to call this method when the Author Component Factory is no longer in use.
   * 
   * For example, if the author component is using a floating license, this method should get called in the applet when the destroy() callback is received
   * in order to quickly release the license to the license server.
   * 
   * @since 13.2
   */
  public void dispose() {
    licenseChecker.releaseFloatingLicense();
    stopWorkspaceAccessPlugins();
    //EXM-25859 And dump options, some plugins may be setting options in our options.
    try {
      Options.getInstance().dumpConfigurationNow();
    } catch (IOException e) {
      logger.error(e, e);
    }
  }
  
  /**
   * Starts the workspace access plugins
   */
  private void startWorkspaceAccessPlugins(){
    AuthorPluginWorkspaceImpl authorCompPluginWorkspace = null;
    // Obtain and iterate the plugins.
    List plugins = PluginManager.getInstance().getPlugins();
    for (Iterator iter = plugins.iterator(); iter.hasNext();) {
      try {
        Plugin plugin = (Plugin) iter.next();
        final PluginDescriptor descriptor = plugin.getDescriptor(); 
        PluginExtension extension = descriptor.getExtension(PluginDescriptor.WORKSPACE_ACCESS);
        if(extension instanceof WorkspaceAccessPluginExtension) {
          if(authorCompPluginWorkspace == null) {
            //Create the plugin workspace implemention.
            pluginsLayoutCustomizer = new AuthorLayoutCustomizer();
            authorCompPluginWorkspace = new AuthorPluginWorkspaceImpl(
                new WorkspaceController(editorManager, ditaMapEditorManager, null, null, null), pluginsLayoutCustomizer);
          }
          final WorkspaceAccessPluginExtension waPlugin = (WorkspaceAccessPluginExtension) extension;
          try {
            //Make sure that the workspace access plugin knows for each customizer the source plugin extension.
            authorCompPluginWorkspace.applicationAboutToStart(descriptor);
            //Notify application started
            waPlugin.applicationStarted(authorCompPluginWorkspace);
          } finally {
            authorCompPluginWorkspace.applicationStarted();
          }
        }
      } catch(final Exception ex) {
        //Plugin loading problem.
        logger.fatal(ex, ex);
      }
    }
  }
  
  /**
   * Stops the workspace access plugins
   */
  private void stopWorkspaceAccessPlugins(){
    // Obtain and iterate the plugins.
    List plugins = PluginManager.getInstance().getPlugins();
    for (Iterator iter = plugins.iterator(); iter.hasNext();) {
      try {
        Plugin plugin = (Plugin) iter.next();
        final PluginDescriptor descriptor = plugin.getDescriptor(); 
        PluginExtension extension = descriptor.getExtension(PluginDescriptor.WORKSPACE_ACCESS);
        if(extension instanceof WorkspaceAccessPluginExtension) {
          final WorkspaceAccessPluginExtension waPlugin = (WorkspaceAccessPluginExtension) extension;
          waPlugin.applicationClosing();
        }
      } catch(final Exception ex) {
        //Plugin loading problem.
        logger.fatal(ex, ex);
      }
    }
  }
  

  /**
   * The author component should try to get the license key from a license server.
   * When the component is created, the license key will be obtained from the license server.
   * 
   * @param host The host of the license server. Never null.
   * @param port The port on which the license server is listening.
   */
  private void registerLicenseServer(LicenseServerInformation serverInfo) {
    LicenseOptions.getInstance().setLicenseServerInformation(serverInfo);
  }

  /**
   * True if initialized
   */
  private boolean initialized = false;

  /**
   * All created components. Used to fire options changes
   */
  private final List<AuthorComponentProvider> allCreatedComponents = new ArrayList<AuthorComponentProvider>();
  
  /**
   * The editor manager
   */
  private AuthorComponentEditorManager editorManager;
  
  /**
   * All created DITA Map components. Used to fire options changes
   */
  private final List<DITAMapTreeComponentProvider> allCreatedDITAMapComponents = new ArrayList<DITAMapTreeComponentProvider>();
  /**
   * The dita map editor manager
   */
  private DITAMapComponentEditorManager ditaMapEditorManager;
  /**
   * The applet code base.
   */
  private URL appletCodeBase;

  /**
   * This method should get called after registering the license.
   * 
   * Initialize the component. Will have effect only once.
   * 
   * @param frameworksZIPURLs The set of ZIPs which contain the frameworks
   * @param optionsZipURL URL to the options ZIP.
   * @param appletCodeBase Code base when run from an applet, can be null when using the author component in a standalone application
   * @param appletID ID when run from Applet, can be null, used to store the frameworks associated with the applet.
   * @param licenseKey The license key to use with the Oxygen Author component
   * @param licenseHost If license key is missing, the license can be floating and the license host and port can be specified, else they are ignored
   * @param licensePort If license key is missing, the license can be floating and the license host and port can be specified, else they are ignored
   * @throws AuthorComponentException
   * 
   * @deprecated Use <code>{@link #init(URL[], URL, URL, String, String, LicenseServerInformation)}</code> or
   *  <code>{@link #init(URL[], URL, URL, String, String)}</code> instead.
   */
  @Deprecated
  public void init(URL[] frameworksZIPURLs, 
      URL optionsZipURL,
      URL appletCodeBase, 
      String appletID, 
      String 
      licenseKey, 
      String licenseHost, 
      int licensePort) throws AuthorComponentException {
    init(frameworksZIPURLs, optionsZipURL, appletCodeBase, appletID, licenseKey, 
        new LicenseServerInformation(
            LicenseServerType.STANDALONE_SERVER, 
            licenseHost, 
            licensePort, 
            null, 
            null, 
            null));
  }

  /**
   * This method should get called after registering the license.
   * 
   * Initialize the component. Will have effect only once.
   * 
   * @param frameworksZIPURLs The set of ZIPs which contain the frameworks
   * @param optionsZipURL URL to the options ZIP.
   * @param appletCodeBase Code base when run from an applet, can be null when using the author component in a standalone application
   * @param appletID ID when run from Applet, can be null, used to store the frameworks associated with the applet.
   * @param licenseKey The license key to use with the Oxygen Author component
   * @throws AuthorComponentException
   */
  public void init(URL[] frameworksZIPURLs, 
      URL optionsZipURL,
      URL appletCodeBase, 
      String appletID, 
      String licenseKey) throws AuthorComponentException {
    init(frameworksZIPURLs, optionsZipURL, appletCodeBase, appletID, licenseKey, null);
  }
  
  
  /**
   * This method should get called after registering the license.
   * 
   * Initialize the component. Will have effect only once.
   * 
   * @param frameworksZIPURLs The set of ZIPs which contain the frameworks
   * @param optionsZipURL URL to the options ZIP.
   * @param appletCodeBase Code base when run from an applet, can be null when using the author component in a standalone application
   * @param appletID ID when run from Applet, can be null, used to store the frameworks associated with the applet.
   * @param serverHost The license key to use with the Oxygen Author component
   * @param serverPort If license key is missing, the license can be floating and the license server information can be specified, else they are ignored
   * @throws AuthorComponentException
   * 
   * @since 12.1
   */
  public void init(URL[] frameworksZIPURLs, 
      URL optionsZipURL,
      URL appletCodeBase, 
      String appletID, 
      String serverHost,
      int serverPort) throws AuthorComponentException {
    init(
        frameworksZIPURLs,
        optionsZipURL,
        appletCodeBase,
        appletID,
        null,
        new LicenseServerInformation(
            LicenseServerType.STANDALONE_SERVER, serverHost, serverPort, null, null, null));
  }
  
  /**
   * This method should get called after registering the license.
   * 
   * Initialize the component. Will have effect only once.
   * 
   * @param frameworksZIPURLs The set of ZIPs which contain the frameworks
   * @param optionsZipURL URL to the options ZIP.
   * @param appletCodeBase Code base when run from an applet, can be null when using the author component in a standalone application
   * @param appletID ID when run from Applet, can be null, used to store the frameworks associated with the applet.
   * @param servletURL The URL to connect to a HTTP license server
   * @param userName User name to connect to the license server.
   * @param password Password to connect to the license server.
   * @throws AuthorComponentException
   * 
   * @since 12.1
   */
  public void init(URL[] frameworksZIPURLs, 
      URL optionsZipURL,
      URL appletCodeBase, 
      String appletID, 
      String servletURL,
      String userName,
      String password) throws AuthorComponentException {
    init(
        frameworksZIPURLs,
        optionsZipURL,
        appletCodeBase,
        appletID,
        null,
        new LicenseServerInformation(
            LicenseServerType.HTTP_SERVER, null, -1, servletURL, userName, password));
  }
  
  /**
   * This method should get called after registering the license.
   * 
   * Initialize the component. Will have effect only once.
   * 
   * @param frameworksZIPURLs The set of ZIPs which contain the frameworks
   * @param optionsZipURL URL to the options ZIP.
   * @param appletCodeBase Code base when run from an applet, can be null when using the author component in a standalone application
   * @param appletID ID when run from Applet, can be null, used to store the frameworks associated with the applet.
   * @param licenseKey The license key to use with the Oxygen Author component
   * @param licenseServerInfo If license key is missing, the license can be floating and the license server information can be specified, else they are ignored
   * @throws AuthorComponentException
   * 
   * @since 12.1
   */
  private void init(URL[] frameworksZIPURLs, 
      URL optionsZipURL,
      URL appletCodeBase, 
      String appletID, 
      String 
      licenseKey, 
      LicenseServerInformation licenseServerInfo) throws AuthorComponentException {
    this.appletCodeBase = appletCodeBase;
    if (!initialized) {
      try {
        //EXM-26773 Make special paste work again.
        MacClipboardUtil.installHTMLClipboardFixForJava1_7();
        //Used in the bundle descriptor
        System.setProperty(OxygenSystemProperties.COM_OXYGENXML_BUNDLE_DESCRIPTOR, "component");
        
        //The preferences directory
        String prefsPath = getPreferencesPathForApplet(appletCodeBase, appletID);

        // Set the name of preferences directory from application descriptor.
        //This is the first thing we MUST do.
        PreferencesDirectoryDetector.setPreferencesDirectoryName(
            BUNDLE_DESCRIPTOR.getPreferencesDirectory() + "/" + prefsPath);
        String prefDirectory = PreferencesDirectoryDetector.getInstance().getPrefsDirectory();
        
        //Download and expand the options ZIP
        File componentPreferencesDirectory = new File(prefDirectory);
        if (optionsZipURL != null) {
          ResourcesSynchronizer optionsSync = new ResourcesSynchronizer(
              componentPreferencesDirectory,
              null);
          optionsSync.sync(new Resource2Synchronize[] {
              new Resource2Synchronize(optionsZipURL)});
          List<ExpandedSyncInfo> si = optionsSync.getSyncManager().getSyncInfo();
          if (si != null && si.size() > 0) {
            ExpandedSyncInfo esi = si.get(0);
            String[] resourceNames = esi.getResourceNames();
            if (resourceNames != null && resourceNames.length > 0) {
              //Fixed options file name.
              if (logger.isDebugEnabled()) {
                logger.debug("Use fixed options file name " + resourceNames[0]);
              }
              Options.setFixedOptionsFileName(resourceNames[0]);
            }
          }
        }
        
        //Skips protocols
        Installer.inhibitInstallProtocols();
        //Author component validator
        ApplicationDescriptorSingleton.setMainFrameDescriptor(new AuthorComponentFrameDescriptor());
        ComponentsValidatorFactory.setIsAuthor(true, true);
        
        //Set the applet frame as the parent frame.
        setAppletFrameAsParentFrame();
        
        if (licenseKey != null) {
          //User based
          registerLicenseKey(licenseKey);
        } else if (licenseServerInfo != null) {
          //Floating
          registerLicenseServer(licenseServerInfo);
        }
        
        // Create the license checker.
        licenseChecker = new LicenseChecker(null, null, BUNDLE_DESCRIPTOR, new FloatingLicenseCallback() {
          @Override
          public void showErrorMessage(IOException ex) {
            if (!fromTests) {
              ApplicationOptionPane.showMessageDialog(
                  "The following error was obtained when trying to license the " + 
                  BUNDLE_DESCRIPTOR.getBundleName() + 
                  ": " + ex.getMessage(), 
                  "Floating License Checking Problem", 
                  ApplicationOptionPane.ERROR_MESSAGE);
            }
          }

          @Override
          public void licenseViolated(int maximum, int count, String connectedUsers) {
            if (!fromTests) {
              ApplicationOptionPane.showMessageDialog(
                  "There are too many users using the " + BUNDLE_DESCRIPTOR.getBundleName() + 
                  " at the same time.\n " +
                  "There are licensed only " + maximum + " users.\n " +
                  "Currently there are connected " + count + " users:\n" + 
                  connectedUsers + "\n", 
                  "Floating License Violation!",
                  ApplicationOptionPane.ERROR_MESSAGE);
            }
          }

          @Override
          public void licenseChanged() {
            // Ignore.
          }

        }) {
          /**
           * @see ro.sync.exml.license.AbstractLicenseChecker#loadLicenseTextFromFile()
           */
          @Override
          protected String loadLicenseTextFromFile() {
            //No fixed license key to be loaded from text file.
            return null;
          }
          /**
           * @see ro.sync.exml.license.LicenseChecker#displayLicenseVersionGreaterThanProduct(java.lang.String)
           */
          @Override
          protected void displayLicenseVersionGreaterThanProduct(String message) {
            //EXM-22402 Ignore this warning, the applet may block.
          }
        };
        //Check the license, fail if not available.
        LicenseInformation licenseInfo = licenseChecker.licenseCheck(!fromTests); 
        if (licenseInfo  == null) {
          throw new AuthorComponentException(
              "ERROR: No valid " + BUNDLE_DESCRIPTOR.getBundleName() + 
              " license found. Please register a valid license key!");
        } else if(!licenseInfo.isEvaluationOnly()) {
          if(licenseKey != null) {
            //A fixed license key was injected
            //EXM-22115
            //Not for evaluation, we accept only site license as an alternative
            if(licenseInfo.getCategory() != null && licenseInfo.getCategory().endsWith("-Site")){
              //Good.
            } else {
              //Bad.
              throw new AuthorComponentException(
                  "ERROR: Only evaluation license keys or site license keys are accepted to be injected in the Author Component.\n" +
              "Please contact support@oxygenxml.com for more details.");
            }
          } else {
            //The user was asked for a license key, any key is accepted.
          }
        }
        
        //EXM-22115 Set the license information
        LicenseInformationProvider.getInstance().setLicenseInformation(licenseInfo);
        //Use a custom frameworks directory
        Options.getInstance().setBooleanProperty(
            OptionTags.USE_CUSTOM_FRAMEWORKS_DIRECTORY_URL,
            true);
        if (frameworksZIPURLs != null && frameworksZIPURLs.length > 0) {
          File newFrameworksDir = new File(componentPreferencesDirectory, FRAMEWORKS_DIR_NAME);
          newFrameworksDir.mkdirs();
          // Synchronize old frameworks with new info
          new ResourcesSynchronizer(newFrameworksDir, null).sync(
              Resource2Synchronize.getResources(frameworksZIPURLs));
          
          // Set the new frameworks directory.
          Options.getInstance().setStringProperty(
              OptionTags.FRAMEWORKS_DIRECTORY_URL_WITH_EDITOR_VARIABLES,
              URLUtil.correct(newFrameworksDir).toString());
          
          //Use this also as the new frameworks dir.
          System.setProperty(OxygenSystemProperties.COM_OXYGENXML_EDITOR_PLUGINS_DIR, newFrameworksDir.getAbsolutePath());
        } else {
          if (logger.isDebugEnabled()) {
            logger.debug("No framework URLs specified");
          }
        }
        
        //Set some default options.
        setDefaultOptions();
        
        //Register the option listeners
        AuthorComponentOptionsForwarder forwarder = new AuthorComponentOptionsForwarder(allCreatedComponents);
        new OptionListeners(forwarder);

        initialized = true;
      } catch(AuthorComponentException ex) {
        throw ex;
      } catch(Exception ex) {
        logger.warn(ex, ex);
        throw new AuthorComponentException(ex);
      }
    }
    
    //The SA Docbook access customizer.
    DocbookAccess.setDocbookAccessCustomizer(new SADocbookAccessCustomizer());
    //The SA common access customizer
    CommonAccess.setCommonAccessCustomizer(new SACommonAccessCustomizer());
    //The DITA Map editor manager
    ditaMapEditorManager = new DITAMapComponentEditorManager(allCreatedDITAMapComponents);
    //The SA DITA Access customizer.
    DITAAccess.setDitaAccessCustomizer(new SADITAAccessCustomizer(ditaMapEditorManager));
    
    //The editor manager
    editorManager = new AuthorComponentEditorManager(allCreatedComponents);
    
    IDEAccess.setInstance(new AuthorComponentIDEAccess());
    ((SimpleSAIDEAccess)IDEAccess.getInstance()).setEditorManager(editorManager);
    //No support for archives.
    ArchiveUtilFacade.setArchiveUtil(new ArchiveUtilAdaptor());
    //EXM-21610 API to control browse CMS from the applet.
    ((SimpleSAIDEAccess)IDEAccess.getInstance()).setURLChooserCustomizersProvider(customizers);
    URLUtil.setRelativeReferencesCustomizersProvider(customizers);
    ((SimpleSAIDEAccess)IDEAccess.getInstance()).setTargetInformationProvider(ditaMapTreeCustomizers);
    //Bind the editor manager.
    FindReplaceDialog.getInstance().setEditorManager(editorManager);
    //Disable the Find All from the Author component.
    FindReplaceDialog.getInstance().setDisableFindAll(true);
    
    //Start the workspace access plugins.
    startWorkspaceAccessPlugins();
  }

  /**
   * Sets the applet frame as the parent frame.
   */
  private void setAppletFrameAsParentFrame() {
    parentFrame = null;
    Frame[] frames = Frame.getFrames();
    if (frames != null && frames.length > 0) {
      parentFrame = frames[0];
      for (int i = 0; i < frames.length; i++) {
        if (frames[i].getClass().getName().contains("PluginEmbeddedFrame")) {
          //Prefer this one, the plugin embedded frame.
          parentFrame = frames[i];
        }
      }
    }
    if (parentFrame != null) {
      //Parent frame for dialogs.
      UnicodeFontManager.getInstance().setParentFrame(parentFrame);
    }
  }
  
  /**
   * Set some default options
   */
  private void setDefaultOptions() {
    //XML Schema 1.0 for now, 1.1 requires additional JARs
    Options.getInstance().setStringProperty(OptionTags.DEFAULT_XML_SCHEMA_VERSION, XMLSchemaVersion.VERSION_1_0.getVersionID());
    Options.getInstance().setIntegerProperty(OptionTags.ASK_OPEN_DITA_MAP_IN_DITAMANAGER, OptionConstants.ALWAYS_NO);
    //Force Hunspell as the spell checker. 
    SpellCheckOptions spellCheckOptions = getSpellCheckOptions();
    spellCheckOptions.preferredChecker = SpellCheckOptions.PREFERRED_CHECKER_HUNSPELL;
    setSpellCheckOptions(spellCheckOptions);
  }
  
  /**
   * Sets a property in the Oxygen preferences.
   * 
   * @param key The key from the Oxygen options 
   * @param value Value for the key.
   */
  public void setObjectProperty(String key, Object value) {
    if(value instanceof Boolean) {
      Options.getInstance().setBooleanProperty(key, (Boolean) value);  
    } else if(value instanceof Double) {
      Options.getInstance().setDoubleProperty(key, (Double) value);  
    } else if(value instanceof File) {
      Options.getInstance().setFileProperty(key, (File) value);  
    } else if(value instanceof int[]) {
      Options.getInstance().setIntArrayProperty(key, (int[]) value);  
    } else if(value instanceof Integer) {
      Options.getInstance().setIntegerProperty(key, (Integer)value);  
    } else if(value instanceof PersistentObject[]) {
      Options.getInstance().setObjectArrayProperty(key, (PersistentObject[]) value);  
    } else if(value instanceof PersistentObject) {
      Options.getInstance().setObjectProperty(key, (PersistentObject) value);  
    } else if(value instanceof String[]) {
      Options.getInstance().setStringArrayProperty(key, (String[]) value);  
    } else if(value instanceof URL) {
      Options.getInstance().setURLProperty(key, (URL) value);  
    } else if(value instanceof String) {
      Options.getInstance().setStringProperty(key, (String) value);  
    }    
  }
  
  /**
   * Creates a new author component
   * @param allowedPages The pages which will be used in the editor. One of the constant fields: 
   * {@link EditorPageConstants#PAGE_TEXT}, {@link EditorPageConstants#PAGE_AUTHOR}, {@link EditorPageConstants#PAGE_GRID}
   * @param initialPage The initial page in which the component will edit.
   *  
   * @return the new author component
   * @throws AuthorComponentException 
   */
  public EditorComponentProvider createEditorComponentProvider(String[] allowedPages, String initialPage) throws AuthorComponentException {
    if(! initialized) {
      throw new AuthorComponentException("Author Component Factory not initialized.\nPlease call the init() method with the right parameters.");
    }
     AuthorComponentProvider authorComponentProvider = new AuthorComponentProvider(appletCodeBase, editorManager, parentFrame, allowedPages, initialPage);
     allCreatedComponents.add(authorComponentProvider);
     return authorComponentProvider;
  }

  /**
   * Creates a new author component
   * 
   * @return the new author component
   * @throws AuthorComponentException 
   * 
   * @deprecated Please use ro.sync.ecss.extensions.api.component.AuthorComponentFactory.createEditorComponentProvider(String[], String) instead.
   */
  @Deprecated
  public AuthorComponentProvider createAuthorComponentProvider() throws AuthorComponentException {
    return (AuthorComponentProvider) createEditorComponentProvider(new String[] {EditorPageConstants.PAGE_AUTHOR}, EditorPageConstants.PAGE_AUTHOR);
  }
  
  /**
   * Creates a new DITA Map tree component provider. Provides access to showing a DITA Map URL in a tree-like fashion (like the DITA Maps Manager view).
   * 
   * @return the new DITA Map tree component provider.
   * @throws AuthorComponentException
   * 
   * @since 14
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p>
   */
  public DITAMapTreeComponentProvider createDITAMapTreeComponentProvider() throws AuthorComponentException {
    if(! initialized) {
      throw new AuthorComponentException("Author Component Factory not initialized.\nPlease call the init() method with the right parameters.");
    }
     DITAMapTreeComponentProvider ditaMapComponentProvider = new DITAMapTreeComponentProvider(ditaMapEditorManager, editorManager, parentFrame);
     allCreatedDITAMapComponents.add(ditaMapComponentProvider);
     return ditaMapComponentProvider;
  }
  
  /**
   * Remove the handlers the factory has to the component used to notify for changes. 
   * It is important to call this method when creating multiple author components
   * in a multiple editor component where editors are sometimes closed. When an editor is closed, this method should get called in order to avoid memory leaks.
   * @param provider The provider to release.
   * 
   * @deprecated Please use ro.sync.ecss.extensions.api.component.AuthorComponentFactory.disposeEditorComponentProvider() instead.
   */
  @Deprecated
  public void disposeAuthorComponentProvider(AuthorComponentProvider provider) {
    disposeEditorComponentProvider(provider);
  }
  
  /**
   * Remove the handlers the factory has to the component used to notify for changes. 
   * It is important to call this method when creating multiple author components
   * in a multiple editor component where editors are sometimes closed. When an editor is closed, this method should get called in order to avoid memory leaks.
   * @param provider The provider to release.
   */
  public void disposeEditorComponentProvider(EditorComponentProvider provider) {
    allCreatedComponents.remove(provider);
    if(editorManager != null) {
      editorManager.componentDisposed((InternalComponentProvider) provider);
    }
  }
  
  /**
   * Remove the handlers the factory has to the component used to notify for changes. 
   * It is important to call this method when creating multiple DITA Map components
   * in a multiple editor component where editors are sometimes closed. When an editor is closed, this method should get called in order to avoid memory leaks.
   * @param provider The provider to release.
   */
  public void disposeDITAMapComponentProvider(DITAMapTreeComponentProvider provider) {
    allCreatedDITAMapComponents.remove(provider);
    if(ditaMapEditorManager != null) {
      ditaMapEditorManager.componentDisposed(provider);
    }
  }
  
  /**
   * If the component is running from an Applet, this method will create a prefix path for storing the frameworks for the applet 
   * @param codeBase The code base of the applet
   * @param appletID The applet ID
   * @return The frameworks prefix.
   */
  static String getPreferencesPathForApplet(URL codeBase, String appletID){
    String prefix = "";
    if(codeBase != null) {
      prefix = FileSystemUtil.proposeValidFileName(codeBase.toString()) + "/";
    }
    if(appletID != null) {
      appletID = FileSystemUtil.proposeValidFileName(appletID);
      prefix += appletID + "/";
    } else {
      prefix += "default/";
    }
    return prefix;
  }
  
  /**
   * Get the spell check options which are currently used for the components.
   * 
   * @return the spell check options which are currently used for the components.
   */
  public SpellCheckOptions getSpellCheckOptions() {
    return (SpellCheckOptions) Options.getInstance().getObjectProperty(
        OptionTags.SPELL_CHECK_OPTIONS);
  }
  
  /**
   * Set the spell check options which are currently used for the components.
   * All options can be changed except the spell checker which is fixed to Hunspell.
   * 
   * @param newSpellCheckOptions The new spell check options to set.
   */
  public void setSpellCheckOptions(SpellCheckOptions newSpellCheckOptions) {
    newSpellCheckOptions.preferredChecker = SpellCheckOptions.PREFERRED_CHECKER_HUNSPELL;
    Options.getInstance().setObjectProperty(
        OptionTags.SPELL_CHECK_OPTIONS, newSpellCheckOptions);
  }
  
  /**
   * Set a handler which will be notified when an URL should be opened.
   * For example the user clicked a link in the Author page.
   * 
   * @param openURLHandler The open URLs handler.
   * 
   * @since 12.2
   */
  public void setOpenURLHandler(OpenURLHandler openURLHandler) {
    editorManager.setOpenURLHandler(openURLHandler);
    ditaMapEditorManager.setOpenURLHandler(openURLHandler);
  }

  /**
   * @see ro.sync.exml.workspace.api.standalone.ReferencesCustomizer#addInputURLChooserCustomizer(ro.sync.exml.workspace.api.standalone.InputURLChooserCustomizer)
   * 
   * @since 13
   */
  @Override
  public void addInputURLChooserCustomizer(InputURLChooserCustomizer inputURLChooserCustomizer) {
    customizers.addInputURLChooserCustomizer(inputURLChooserCustomizer);
  }

  /**
   * @see ro.sync.exml.workspace.api.standalone.ReferencesCustomizer#addRelativeReferencesResolver(java.lang.String, ro.sync.exml.workspace.api.util.RelativeReferenceResolver)
   * 
   * @since 13
   */
  @Override
  public void addRelativeReferencesResolver(String protocol, RelativeReferenceResolver resolver) {
    customizers.addRelativeReferencesResolver(protocol, resolver);
  }
  
  /**
   * Add a provider which can resolve certain information for each topic ref, without the component needing to parse that topic reference.
   * @param protocol The protocol for which the provider is registered
   * @param targetInformationProvider The topic reference target information provider provider.
   * 
   * @since 14
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   */
  public void addDITAMapTreeTargetInformationProvider(String protocol, TopicRefTargetInfoProvider targetInformationProvider) {
    ditaMapTreeCustomizers.addTopicRefTargetInformationProvider(protocol, targetInformationProvider);
  }
  
  /**
   * Set a fixed key for licensing the MathFlow editor dialog used to edit embedded MathML equations.
   * @param fixedKey  The fixed key. The key needs to be obtained from MathFlow:
   * 
   * http://dessci.com/
   * 
   * and has the following format:
   * 
   * MFSCKKK-KKKKKK-KKKKK
   * 
   * If no editor key will be given then MathFlow will be used neither for editing nor for rendering.
   * 
   * @since 14
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   */
  public void setMathFlowFixedLicenseKeyForEditor(String fixedKey) {
    MathFlowMathMLImageHandler.setMathFlowFixedLicenseKeyForEditor(fixedKey);
  }
  
  /**
   * Set a fixed key for licensing the MathFlow composer used to view embedded MathML equations.
   * @param fixedKey  The fixed key. The key needs to be obtained from MathFlow:
   * 
   * http://dessci.com/
   * 
   * and has the following format:
   * 
   * MFSEKKK-KKKKKK-KKKKK
   * 
   * If no composer key will be given then the fallback for rendering will be the Apache JEuclid library.
   * 
   * @since 14
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   */
  public void setMathFlowFixedLicenseKeyForComposer(String fixedKey) {
    MathFlowMathMLImageHandler.setMathFlowFixedLicenseKeyForComposer(fixedKey);
  }
  
  /**
   * Access to XML utilities.
   * 
   * @return Access to XML utilities.
   * 
   * @since 14.2
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   * 
   */
  public XMLUtilAccess getXMLUtilAccess() {
    if(accessUtil == null) {
      accessUtil = new AccessUtilImpl();
    }
    return accessUtil;
  }
  
  /**
   * Get access to utility methods.
   * 
   * @return access to utility methods.
   * 
   * @since 14.2
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   */
  public UtilAccess getUtilAccess() {
    if(accessUtil == null) {
      accessUtil = new AccessUtilImpl();
    }
    return accessUtil;
  }
  
  /**
   * Get the toolbar customizers which were added by all installed plugins.
   * 
   * @return Returns the toolbar customizers which were added by all installed plugins.
   * 
   * @since 14.2
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p>
   */
  public List<ToolbarComponentsCustomizer> getPluginToolbarCustomizers() {
    return pluginsLayoutCustomizer != null ? pluginsLayoutCustomizer.getToolbarCustomizers() : null;
  }
  
  /**
   * Get the view customizers which were added by all the installed plugins.
   * 
   * @return Returns the view customizers which were added by all installed plugins.
   * 
   * @since 14.2
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p>
   */
  public List<ViewComponentCustomizer> getPluginViewCustomizers() {
    return pluginsLayoutCustomizer != null ? pluginsLayoutCustomizer.getViewCustomizers() : null;
  }
  
  /**
   * Get the menu bar customizers which were added by all the installed plugins.
   * 
   * @return Returns the menu bar customizers which were added by all installed plugins.
   * 
   * @since 14.2
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p>
   */
  public List<MenuBarCustomizer> getPluginMenubarCustomizers() {
    return pluginsLayoutCustomizer != null ? pluginsLayoutCustomizer.getMenubarCustomizers() : null;
  }
}
