/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component.ditamap;

import java.net.URL;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.dita.mapeditor.DITAMapEditor;
import ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager;
import ro.sync.ecss.dita.reference.keyref.KeyInfo;
import ro.sync.ecss.extensions.api.component.BaseComponentEditorManager;
import ro.sync.exml.editor.AbstractEditor;

/**
 * The DITA Map component editor manager
 */
@API(type=APIType.INTERNAL, src=SourceType.PRIVATE)
public class DITAMapComponentEditorManager extends BaseComponentEditorManager implements DITAMapsEditorManager { 
  
  /**
   * Constructor.
   *
   * @param allProviders All component providers.
   */
  public DITAMapComponentEditorManager(List<DITAMapTreeComponentProvider> allProviders) {
    super(allProviders);
  }
  
  /**
   * @see ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager#getAllDITAMapsReferencedURLs()
   */
  @Override
  public Set<URL> getAllDITAMapsReferencedURLs() {
    return null;
  }

  /**
   * @see ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager#getCurrentDITAMapReferencedUrls()
   */
  @Override
  public Set<URL> getCurrentDITAMapReferencedUrls() {
    return null;
  }

  /**
   * @see ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager#saveCurrentEditor()
   */
  @Override
  public void saveCurrentEditor() {
    //
  }

  /**
   * @see ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager#setEnableLinkWithEditor(boolean)
   */
  @Override
  public void setEnableLinkWithEditor(boolean enableLinkWithEditor) {
    //
  }

  /**
   * @see ro.sync.ecss.dita.mapeditor.DITAMapsEditorManager#setShowDITAMapsManagerOnOpen(boolean)
   */
  @Override
  public void setShowDITAMapsManagerOnOpen(boolean enable) {
    //
  }

  /**
   * @see ro.sync.ecss.dita.reference.keyref.KeysManager#getContextDITAMapURL()
   */
  @Override
  public URL getContextDITAMapURL() {
    AbstractEditor selectedEditor = getSelectedEditor();
    if(selectedEditor != null) {
      return selectedEditor.getEditorLocation();
    }
    return null;
  }

  /**
   * @see ro.sync.ecss.dita.reference.keyref.KeysManager#getKeys(URL)
   */
  @Override
  public LinkedHashMap<String, KeyInfo> getKeys(URL originatorURL) {
    LinkedHashMap<String, KeyInfo> keys = new LinkedHashMap<String, KeyInfo>();
    AbstractEditor selectedEditor = getSelectedEditor();
    if (selectedEditor instanceof DITAMapEditor) {
      keys = ((DITAMapEditor)selectedEditor).gatherKeys();
    }
    return keys;
  }
}