/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component.ditamap;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.io.File;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JComponent;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.document.Descriptor;
import ro.sync.ecss.dita.mapeditor.DITAMapEditor;
import ro.sync.ecss.dita.mapeditor.DITAMapEditorPage;
import ro.sync.ecss.extensions.api.ExtensionsBundle;
import ro.sync.ecss.extensions.api.component.AuthorComponentException;
import ro.sync.ecss.extensions.api.component.InternalComponentProvider;
import ro.sync.ecss.extensions.api.component.listeners.AuthorComponentListener;
import ro.sync.ecss.extensions.api.component.listeners.DITAMapTreeComponentListener;
import ro.sync.ecss.persistance.AuthorExtensionPO;
import ro.sync.ecss.persistance.CSSDescriptorPO;
import ro.sync.exml.Tags;
import ro.sync.exml.editor.AbstractEditor;
import ro.sync.exml.editor.BasicEditor;
import ro.sync.exml.editor.EditorListener;
import ro.sync.exml.editor.EditorManager;
import ro.sync.exml.editor.EditorPage;
import ro.sync.exml.editor.EditorPageChangeEvent;
import ro.sync.exml.editor.EditorPageConstants;
import ro.sync.exml.editor.InitialPageHint;
import ro.sync.exml.editor.persistance.DocumentTypePO;
import ro.sync.exml.editor.status.StatusModelListener;
import ro.sync.exml.editor.status.StatusPanel;
import ro.sync.exml.editor.xmleditor.DetectionTaskListener;
import ro.sync.exml.editor.xmleditor.DocumentTypeProviderListener;
import ro.sync.exml.editor.xmleditor.XmlEditor;
import ro.sync.exml.workspace.api.editor.WSEditor;
import ro.sync.exml.workspace.api.editor.page.ditamap.DITAMapPopupMenuCustomizer;
import ro.sync.exml.workspace.api.editor.page.ditamap.WSDITAMapEditorPage;
import ro.sync.exml.workspace.impl.editor.WSEditorImpl;
import ro.sync.licensemanager.LicenseInformation;
import ro.sync.licensemanager.LicenseInformationProvider;
import ro.sync.ui.application.action.AccelAction;
import ro.sync.util.URLUtil;

/**
 * A component encapsulating editing a DITA Map in a DITA Maps Manager tree-like structure.
 * 
 * @since 14
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p>
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PRIVATE)
public class DITAMapTreeComponentProvider extends InternalComponentProvider {
  /** 
   * Logger for logging. 
   */
  private static final Logger logger = Logger.getLogger(DITAMapTreeComponentProvider.class.getName());
  
  /**
   * The component listeners
   */
  private final List<DITAMapTreeComponentListener> componentListeners =
    new ArrayList<DITAMapTreeComponentListener>();

  /**
   * The editor.
   */
  private final XmlEditor editor;
  
  /**
   * The status panel
   */
  private final StatusPanel statusPanel = new StatusPanel(true, true);

  /**
   * True if the detection finished
   */
  protected boolean detectionFinished = false;
  
  /**
   * Get access to the WS Editor.
   */
  private WSEditor editorAccess;

  /**
   * Save the content back to the original URL from where it was loaded using the internal support. 
   * Useful only when you provide an initial URL from which the component is loaded.
   */
  public void save() {
    getWSEditorAccess().save();
  }
  
  /**
   * Untitled files counter
   */
  private static int untitledFilesCounter = 0;
  
  /**
   * Sets the content to edit.
   * <p>This does not guarantee that the set content has been interpreted, you should set an
   * {@link AuthorComponentListener} and listen for <code>documentTypeChanged()</code> before using
   * the author extension actions.
   * 
   * @param url     URL to load, can be <code>null</code> if the reader is specified
   *                If no XML content reader is given, the URL will be used both to obtain the
   *                content and to solve relative references (eg: images).
   *                If the XML content reader is also given, the URL will only be used to solve
   *                relative references from the file.
   * @param reader  The reader.
   * 
   * @throws AuthorComponentException When there was a load problem (eg: IOException).
   */
  public void load(URL url, Reader reader) throws AuthorComponentException{
    try {
      detectionFinished = false;
      editor.getDocumentTypeProvider().setDetectionTaskListener(new DetectionTaskListener() {
        @Override
        public void detectionTaskFinished() {
          detectionFinished = true;
        }
      });
      if (reader == null && url != null) {
        //Show the file status
        statusPanel.setShowFilePath(true);
        editor.open(url);
        //Request focus in the opened editor
        editor.requestFocusInEditor();
      } else if (reader != null) {
        //Do not show the file status
        statusPanel.setShowFilePath(false);
        if (url == null) {
          url = URLUtil.correct(new File("Untitled" + (++untitledFilesCounter) + ".xml"));
        }
        editor.setLoadedURL(url);
        editor.readDocument(reader, url, true);
        //Request focus in the opened editor
        editor.requestFocusInEditor();
      } else {
        throw new AuthorComponentException("Both URL and Reader are NULL");
      }
      
      //Fire loaded document changed
      for (int i = 0; i < componentListeners.size(); i++) {
        componentListeners.get(i).loadedDocumentChanged();
      }
    } catch (Exception e) {
      e.printStackTrace();
      throw new AuthorComponentException(e);
    }
  }
  
  /**
   * Constructor.
   * @param parentEditorManager The parent editor manager
   * @param mainFileOpener  The main file opener
   * @param parentFrame  The parent frame.
   * @throws AuthorComponentException 
   */
  public DITAMapTreeComponentProvider(final DITAMapComponentEditorManager parentEditorManager, EditorManager mainFileOpener, Frame parentFrame)
  throws AuthorComponentException {
    if (logger.isDebugEnabled()) {
      logger.debug("Create DITA Map tree component");
    }
    editor = new DITAMapEditor(parentFrame, parentEditorManager, mainFileOpener) {
      /**
       * @see ro.sync.exml.editor.AbstractEditor#setSelectedBorder(boolean)
       */
      @Override
      protected void setSelectedBorder(boolean isSelected) {
        //Show border only if the license is for evaluation purposes
        LicenseInformation li = LicenseInformationProvider.getInstance().getLicenseInformation();
        if (li != null && li.isEvaluationOnly()) {
          super.setSelectedBorder(isSelected);
        }
      }
      /**
       * @see ro.sync.exml.editor.xmleditor.XmlEditor#createEditorPages()
       */
      @Override
      protected List<EditorPage> createEditorPages() {
        //Only DITA Map page available for now.
        List<EditorPage> pages = new ArrayList();
        ditaMapPage = new DITAMapEditorPage(this, parentEditorManager); 
        pages.add(ditaMapPage);
        return pages;
      }
      /**
       * @see ro.sync.exml.editor.xmleditor.AbstractXmlEditor#getInitialPage()
       */
      @Override
      protected String getInitialPage() {
        return EditorPage.PAGE_DITA_MAP;
      }
    };
    editor.setInitialPage(new InitialPageHint(EditorPageConstants.PAGE_DITA_MAP, true));
    editor.setShowPagesSwitch(false);
    editor.initFrame();
    //Do not use the lazy bounds.
    editor.setEnableSetBoundsInEditor(true);
    //Start from a dummy URL, avoid NPE if user does not set any content
    try {
      editor.setLoadedURL(URLUtil.correct(new File("Untitled.ditamap")));
    } catch (MalformedURLException e) {
      //
    }
    //Fire the component listeners
    editor.addEditorListener(
        new EditorListener() {
          @Override
          public void modifiedStateChanged(Descriptor editor) {
            for (int i = 0; i < componentListeners.size(); i++) {
              componentListeners.get(i).modifiedStateChanged(isModified());
            }
          }
          @Override
          public void loadedFileChanged(Descriptor editor) {
            //
          }
          @Override
          public void nameChanged(BasicEditor editor) {
            //
          }
          @Override
          public void editorPageChanged(EditorPageChangeEvent event) {
            //
          }
          @Override
          public void editorLoaded(AbstractEditor editor) {
            //
          }
          @Override
          public void editorClosed(AbstractEditor editor) {
            // Not needed.
          }
        });
    
    //Fire to the component listeners
    editor.getDocumentTypeProvider().addDTListener(
        new DocumentTypeProviderListener() {
          @Override
          public void documentTypeChanged(
              DocumentTypePO documentType, ExtensionsBundle bundle, long detectTime) {
            for (int i = 0; i < componentListeners.size(); i++) {
              componentListeners.get(i).documentTypeChanged();
            }
          }

          @Override
          public void cssChanged(CSSDescriptorPO[] cssDescriptors, long detectTime) {
            for (int i = 0; i < componentListeners.size(); i++) {
              componentListeners.get(i).documentTypeChanged();
            }
          }

          @Override
          public void authorExtensionChanged(
              AuthorExtensionPO authorExtension,
              ExtensionsBundle bundle,
              String[] classpath,
              DocumentTypePO documentType,
              long detectionTime) {
            for (int i = 0; i < componentListeners.size(); i++) {
              componentListeners.get(i).documentTypeChanged();
            }
          }
        });
    editor.getStatusModel().addStatusModelListener(statusPanel);
    
    //Now register the keyboard shortcuts for these common actions
    Map<String, AbstractAction> ditaMapCommonActions = getDITACommonActions();
    Iterator<AbstractAction> iter = ditaMapCommonActions.values().iterator();
    while(iter.hasNext()) {
      AbstractAction action = iter.next();
      if (action instanceof AccelAction) {
        ((AccelAction)action).putInActionMap((JComponent) getDITAAccess().getDITAMapTreeComponent());
      }
    }
    
    //Notify the last focused component
    ((JComponent)getDITAAccess().getDITAMapTreeComponent()).addFocusListener(
        new FocusAdapter() {
          /**
           * @see java.awt.event.FocusAdapter#focusGained(java.awt.event.FocusEvent)
           */
          @Override
          public void focusGained(FocusEvent e) {
            parentEditorManager.setLastFocusedComponent(DITAMapTreeComponentProvider.this);
          }
        });

    //Fire the editor selected from the beginning. Useful to show the evaluation border
    editor.editorSelected(true, false);
    
    //The editor access
    editorAccess = new WSEditorImpl(editor, editor);
  }

  /**
   * Create a reader over the editor's current page content
   * 
   * @return The reader over the current page's content
   */
  public Reader createReader() {
    return editor.createReader(); 
  }
  
  /**
   * Adds a component listener.
   * 
   * @param listener The listener.
   */
  public void addDITAMapTreeComponentListener(DITAMapTreeComponentListener listener) {
    if (!componentListeners.contains(listener)) {
      componentListeners.add(listener);
    }
  }
  
  /**
   * Removes a component listener.
   * 
   * @param listener The listener.
   */
  public void removeAuthorComponentListener(DITAMapTreeComponentListener listener) {
    componentListeners.remove(listener);
  }  
  
  /**
   * @return The editor panel.
   */
  public Component getEditorComponent() {
    return editor.getComponent();
  }
  
  /**
   * @return The status panel.
   */
  public Component getStatusComponent() {
    return statusPanel;
  }
  
  /**
   * Get the map of DITA Map common actions (undo, redo, cut, copy, paste, etc).
   * 
   * @return The map with (action id, AbstractAction) pairs with the actions defined for working in
   *         the DITA Map.
   */
  public Map<String, AbstractAction> getDITACommonActions(){
    Map<String, AbstractAction> allCommonActions = getDITAPage().getAllActions();
    allCommonActions.remove(Tags.DITA_MAPS_MANAGER + "/" + Tags.APPLY_STYLESHEET_TOOLTIP);
    allCommonActions.remove(Tags.DITA_MAPS_MANAGER + "/" + Tags.CONFIGURE_TRANSFORMATION);
    allCommonActions.remove(Tags.DITA_MAPS_MANAGER + "/" + Tags.LINK_WITH_EDITOR);
    allCommonActions.remove(Tags.DITA_MAPS_MANAGER + "/" + Tags.VALIDATE_AND_CHECK_FOR_COMPLETENESS);
    return allCommonActions;
  }
  
  /**
   * @return <code>true</code> if the component is modified.
   */
  public boolean isModified() {
    return editor.getModifiedStatus() == StatusModelListener.MODIFIED;
  }
  
  /**
   * Sets the modified status.
   * 
   * @param modified <code>true</code> to flag as modified.
   */
  public void setModified(boolean modified) {
    getWSEditorAccess().setModified(modified);
  }
  
  /**
   * Get the access to the WS Editor.
   *
   * @return The editor access.
   */
  public WSEditor getWSEditorAccess() {
    return editorAccess;
  }
  
  /**
   * @return The editor.
   */
  @Override
  protected AbstractEditor getEditorKey() {
    return editor;
  }
  
  /**
   * Get the page access used to perform various operations on the DITA Map tree.
   *
   * @return The page access.
   */
  public WSDITAMapEditorPage getDITAAccess() {
    return (WSDITAMapEditorPage) ((DITAMapEditorPage)editor.getEditorPage(EditorPage.PAGE_DITA_MAP)).getPageAccess();
  }
  
  /**
   * Get the DITA Map page access used to perform various operations on the Author Page.
   *
   * @return The author page.
   */
  DITAMapEditorPage getDITAPage() {
    return (DITAMapEditorPage)editor.getEditorPage(EditorPage.PAGE_DITA_MAP);
  }
  
  /**
   * The Pop-up customizer can be used to add/remove actions from the pop-up menu in the DITA Map tree
   * editor before showing it. If everything is removed then the menu will not be shown.
   * 
   * @param popUpCustomizer The pop Up Customizer.
   */
  public void setEditorPopUpCustomizer(DITAMapPopupMenuCustomizer popUpCustomizer) {
    getDITAAccess().setPopUpMenuCustomizer(popUpCustomizer);
  }
  
  /**
   * Print the DITA Map component content. Shows the Print dialog.
   * 
   * @param preview <code>true</code> to show the Print Preview dialog, <code>false</code> to show the Print dialog.
   */
  public void print(boolean preview) {
    editor.print(preview);
  }
}