/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.highlights;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorDocumentController;


/**
 * Listener for all the events related to the {@link AuthorPersistentHighlight}.
 * 
 * You can register such a listener using 
 * {@link AuthorDocumentController#addAuthorPersistentHighlightListener(AuthorPersistentHighlightsListener)}, 
 * 
 * @since 14
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p>   
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class AuthorPersistentHighlightsListener {

  /**
   * Notified when a {@link AuthorPersistentHighlight} is added to the document.
   * 
   * @param highlight Added highlight.
   */
  public abstract void highlightAdded(AuthorPersistentHighlight highlight);
  
  /**
   * Notified when a {@link AuthorPersistentHighlight} is removed from the document.
   * 
   * @param highlight The removed highlight.
   */
  public abstract void highlightRemoved(AuthorPersistentHighlight highlight);
  
  /**
   * Notified when a property of a {@link AuthorPersistentHighlight} is updated, 
   * for example changing the comment of a change tracking marker.  
   * 
   * 
   * @param highlight The updated highlight.
   */
  public abstract void highlightUpdated(AuthorPersistentHighlight highlight);
  
  /**
   * Event which notifies that the list of highlights (change tracking, comments or custom) has changed in an unpredictable way.
   * 
   * API code which inserts or deletes multiple fragments in one operation like:<br>
   * 
   * ro.sync.ecss.extensions.api.AuthorDocumentController.insertMultipleFragments(AuthorElement, AuthorDocumentFragment[], int[])<br>
   * ro.sync.ecss.extensions.api.AuthorDocumentController.insertMultipleElements(AuthorElement, String[], int[], String)<br>
   * ro.sync.ecss.extensions.api.AuthorDocumentController.multipleDelete(AuthorElement, int[], int[])<br>
   * 
   * will not fire atomic events each time a highlight is reconfigured.<br>
   * Instead, they will fire a single event after the operation has finished notifying the listener to reconfigure all its highlight data.<br>
   * 
   * This event will follow an:<br>
   * 
   *  ro.sync.ecss.dom.AuthorDocumentListener.authorNodeStructureChanged(AuthorNodeStructureChangedEvent)<br>
   *  
   *  event.
   */
  public abstract void highlightsChanged();
}
