/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.editor.page.ditamap;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.OptionsStorage;
import ro.sync.ecss.extensions.api.attributes.AuthorAttributesDisplayFilter;
import ro.sync.exml.workspace.api.editor.page.WSEditorPage;

/**
 * DITA Maps Manager editor page. 
 * 
 * @since 12.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface WSDITAMapEditorPage extends WSEditorPage {

  /**
   * Returns the DITA Map document controller. It has methods for changing the document model.
   * 
   * @return The controller for DITA Map structure. Cannot be <code>null</code>.
   */
  AuthorDocumentController getDocumentController();
  
  /**
   * The object that manages the options stored for DITA Map extensions. 
   * This is also responsible for adding and removing listeners that are notified
   * about the option changes.
   * 
   * @return The object that manages the options stored for DITA Map extensions.
   */
  OptionsStorage getOptionsStorage();
  
  /**
   * Set the pop-up menu customizer which can be used to customize the pop-up menu
   * (add/remove actions) before showing it in the DITA Mapa Manager page.
   *
   * @param popUpCustomizer the pop-up menu customizer.
   */
  void setPopUpMenuCustomizer(DITAMapPopupMenuCustomizer popUpCustomizer);
  
  /**
   * Get the internal component on which the DITA Map page is rendered (a javax.swing.JTree).
   * 
   * @return for the stand alone version, a javax.swing.JTree, 
   * for the Oxygen plugin for Eclipse an instance of TreeViewer 
   */
  Object getDITAMapTreeComponent();
  
  /**
   * Adds a filter for displaying attributes to the current DITA Map Tree. The filter will be applied when editing the attributes for a topic reference.
   * 
   * @param attributesDisplayFilter The {@link AuthorAttributesDisplayFilter} to be added.
   * 
   * @since 13.1
   */
  void addAuthorAttributesDisplayFilter(AuthorAttributesDisplayFilter attributesDisplayFilter);
  
  /**
   * Remove a filter for displaying attributes to the current DITA Map Tree.
   * 
   * @param attributesDisplayFilter The {@link AuthorAttributesDisplayFilter} to be removed.
   * 
   * @since 13.1
   */
  void removeAuthorAttributesDisplayFilter(AuthorAttributesDisplayFilter attributesDisplayFilter);
  
  /**
   * Sets the specified flag to indicate whether or not the DITA Map page should be editable.
   *
   * @param editable <code>true</code> if the DITA Map page should be editable.
   * 
   * @since 13.1
   */
  public void setEditable(boolean editable);
}