/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2011 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.node.customizer;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Provide information like node name, node namespace, attributes (if available), that will be used 
 * for Author outline, Author bread crumb, Text page outline, content completion proposals window 
 * or DITA Map view rendering customization.
 * 
 * @author Iulian Velea
 * @author Alina Iordache
 * 
 * @since 13.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public abstract class NodeRendererCustomizerContext {
  /**
   * Get the node name.
   * 
   * @return Returns the node name.
   */
  public abstract String getNodeName();
  
  /**
   * Get the node namespace.
   * 
   * @return Returns the node namespace.
   */
  public abstract String getNodeNamespace();
  
  /**
   * Returns the number of the element attributes.
   * 
   * @return The number of the element attributes.
   */
  public abstract int getAttributesCount();
  
  /**
   * Get the qualified attribute name at index.
   * 
   * @param index Index of the attribute.
   * @return The qualified attribute name at index.
   */
  public abstract String getAttributeQName(int index);
  
  /**
   * Get attribute value for given attribute qualified name.
   * 
   * @param attrQName The qualified attribute name.
   * @return The attribute value.
   */
  public abstract String getAttributeValue(String attrQName);
  
  /**
   * Get namespace URI for a given attribute qualified name.
   * 
   * @param attrQName The attribute qualified name.
   * @return The attribute namespace URI.
   */
  public abstract String getAttributeNamespace(String attrQName);
}